package sushi_sample;


/**
 * 回転寿司のレーン
 */
public class SushiLane {

	/** すしレーンの描画位置 */
	private int laneX, laneY;
	/** すしレーンの大きさ（皿の枚数分） */
	private int laneWidth, laneHeight;
	/** 皿の大きさ */
	private int dishSize;
	
	private Dish[] dishes;
	
	public SushiLane(int laneX, int laneY, int laneWidth, int laneHeight, int dishSize) {
		this.laneX = laneX;
		this.laneY = laneY;
		this.laneWidth = laneWidth;
		this.laneHeight = laneHeight;
		this.dishSize = dishSize;
		
		int amount = laneWidth * 2 + (laneHeight - 2) * 2;
		dishes = new Dish[amount];
		
		// すし皿の位置の初期化
		int counter = 0;
		for (int i = 0; i < laneHeight; i++) {
			dishes[counter++] = new Dish(laneX, laneY + i * dishSize);
		}
		for (int i = 1; i < laneWidth-1; i++) {
			dishes[counter++] = new Dish(laneX + i * dishSize, laneY + dishSize * (laneHeight - 1));
		}
		for (int i = laneHeight - 1; i >= 0; i--) {
			dishes[counter++] = new Dish(laneX + dishSize * (laneWidth - 1), laneY + i * dishSize);
		}
		for (int i = laneWidth - 2; i > 0; i--) {
			dishes[counter++] = new Dish(laneX + i * dishSize, laneY);
		}
		
		new Rotator().start(); // レーンの回転開始
	}
	
	/** 
	 * 空いてる場所にすしを追加する。 
	 * @throws InterruptedException 
	 */
	public synchronized void addSushi(Sushi sushi) throws InterruptedException {
		//以下にプログラムを記述する
		
		// 1. 空きができるまで待つ
		
		// 2. 空いている皿にすしを置く
		
		// すしが追加されるまで待つ。
		
	}
	
	/** レーンからすしを取る。 
	 * @throws InterruptedException */
	public synchronized Sushi takeDish() throws InterruptedException {
		Sushi sushi = null;
		//以下にプログラムを記述する
		
		// 1. すしが追加されるまで待つ
		
		// 2. 皿からすしを取る
		
		return sushi;
	}
	
	/** レーン上にあるすしの数を返す。 */
	private synchronized int getNumOfSushi() {
		int num = 0;
		// 必要であれば、このメソッドも利用する
		return num;
	}
	
	/** お皿の配列のコピーを返す。 */
	public synchronized Dish[] getCopyOfDishes() {
		Dish[] copyOfDishes = new Dish[dishes.length];
		for (int i = 0; i < dishes.length; i++) {
			copyOfDishes[i] = new Dish(dishes[i]);
		}
		
		return copyOfDishes;
	}
	
	/** すしレーンを回転させる。 */
	private synchronized void rotate() {
		final int distance = 2; // 移動距離
		
		for (Dish dish : dishes) {
			int x = dish.getX();
			int y = dish.getY();
		
			// 上のレーンなら
			if (y== laneY) {
				// 右上なら下へ方向転換
				if (x == laneX + dishSize * (laneWidth - 1)) {
					y += distance;
				} else {
					x += distance; // 右に進む
				}
			}
			// 右のレーンなら
			else if (x == laneX + dishSize * (laneWidth - 1)) {
				// 右下なら左へ方向転換
				if (y == laneY + dishSize * (laneHeight - 1)) {
					x -= distance;
				} else {
					y += distance; // 下に進む
				}
			}
			// 下のレーンなら
			else if (y == laneY + dishSize * (laneHeight - 1)) {
				// 左下なら上へ方向転換
				if (x == laneX) {
					y -= distance;
				} else {
					x -= distance; // 左に進む
				}
			}
			// 左のレーンなら
			else if (x == laneX) {
				// 左上なら右へ方向転換
				if (y == laneY) {
					x += distance;
				} else {
					y -= distance; // 上に進む
				}
			}
			
			dish.setX(x);
			dish.setY(y);
		}
	}
	
	/** レーンを回転するスレッドクラス */
	private class Rotator extends Thread {
		
		public void run() {
			try {
				while (true) {
					rotate();
					sleep(50);
				}
			} catch (InterruptedException e) {}
		}
	}
}
