package sushi_sample;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

import javax.swing.*;


@SuppressWarnings("serial")
/**
 * 回転寿司のアニメーションの表示を行うクラス
 */
public class SushiPanel extends JPanel implements ActionListener{

	// すしレーンの描画位置
	private int laneX = 20;
	private int laneY = 20;
	// すしレーンの大きさ（皿の枚数分）
	private int laneWidth = 6;
	private int laneHeight = 4;
	// 皿の大きさ
	private int dishSize = 60;
	
	// このパネルの所有クラス 画像の読み込み方法の判定に使用
	private Component parent;
	
	private SushiLane sushiLane;
	
	public SushiPanel(Component parent) {
		try{
			// UIをOSのものに変更
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}catch(Exception e){}
		
		this.parent = parent;
		
		// すしレーン起動
		sushiLane = new SushiLane(laneX, laneY, laneWidth, laneHeight, dishSize);
		
		// すし職人を生成
		
		
		//　ボタン
		JButton btn1 = new JButton(" ボタン１ ");
		btn1.setActionCommand("b1");
		btn1.setFont(new Font(/*Font.DIALOG*/"Dialog", Font.PLAIN, 16));
		btn1.setFocusPainted(false);
		btn1.addActionListener(this);
		
		SushiCanvas canvas = new SushiCanvas();
		JPanel panel = new JPanel();
		panel.add(btn1);
		
		this.setLayout(new BorderLayout());
		this.add(canvas, BorderLayout.CENTER);
		this.add(panel, BorderLayout.SOUTH);
		
		new Thread(canvas).start();
	}
	
	private class SushiCanvas extends Canvas implements Runnable {
		
		// キャンバスの大きさ
		private int canvasWidth;
		private int canvasHeight;
		
		// ダブルバッファ
		private BufferedImage bufferedImage;
		private Graphics graphicBuffer;
		private Map<String, Image> images;
		
		private SushiCanvas() {
			canvasWidth =  laneX * 2 + dishSize * laneWidth;
			canvasHeight = laneY + 2 + dishSize * (laneHeight + 1);
			
			// ダブルバッファ
			this.setPreferredSize(new Dimension(canvasWidth, canvasHeight));
			bufferedImage = new BufferedImage(canvasWidth, canvasHeight, BufferedImage.TYPE_3BYTE_BGR);
			graphicBuffer = bufferedImage.getGraphics();
			
			// 画像読み込み
			final String[] neta = {"anago", "ebi", "hotate", "ika", "ikura", "maguro", "tako", "tamago", "tekka", "uni"};
			images = new HashMap<String, Image>();
			
			if (parent instanceof JFrame) {
				Toolkit toolkit = Toolkit.getDefaultToolkit();
				for (String s : neta) {
					images.put(s, toolkit.createImage("./resource/" + s + ".gif"));
				}
			} else if (parent instanceof JApplet) {
				JApplet applet = (JApplet) parent;
				for (String s : neta) {
					images.put(s, applet.getImage(applet.getCodeBase(), "./resource/" + s + ".gif"));
				}
			}
		}
		
		public void run() {
			while (true) {			
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						repaint();
					}
				});
				
				try {		
					Thread.sleep(100);
				} catch (InterruptedException e) {}
			}
		}
		
		public void update(Graphics g) {
			paint(g);
		}
		
		public void paint(Graphics g) {
			// 背景
			graphicBuffer.setColor(new Color(0xf5, 0xde, 0xb3));
			graphicBuffer.fillRect(0, 0, canvasWidth, canvasHeight);
			
			// 枠
			graphicBuffer.setColor(new Color(0xde, 0xb8, 0x87));
			graphicBuffer.fillRoundRect(laneX, laneY, dishSize*6, dishSize*4, dishSize, dishSize);
			graphicBuffer.setColor(new Color(0xf5, 0xde, 0xb3));
			graphicBuffer.fillRoundRect(dishSize + laneX, dishSize + laneY, dishSize*4, dishSize*2, dishSize/2, dishSize/2);
			
			// 職人
			graphicBuffer.setFont(new Font(/*Font.DIALOG*/"Dialog", Font.BOLD, 28));
			graphicBuffer.setColor(Color.BLUE);
			graphicBuffer.drawString("職人1", 100, 140);
			graphicBuffer.setColor(Color.RED);
			
			// 客席
			graphicBuffer.setColor(new Color(0xd2, 0xb4, 0x8c));
			graphicBuffer.fillOval(35, 270, 50, 50);
			
			// お客さん			
			// 以下に記述する
			
			//　皿
			graphicBuffer.setColor(Color.WHITE);
			Dish[] dishes = sushiLane.getCopyOfDishes();
			for (Dish dish : dishes) {
				int x = dish.getX();
				int y = dish.getY();
				Sushi sushi = dish.getSushi();
				
				// 回っている皿
				graphicBuffer.fillOval(x, y, dishSize, dishSize); 
				if (sushi != null) {
					// 回っているすし
					graphicBuffer.drawImage(images.get(sushi.getName()), x+10, y+10, null);
				}
			}
			
			g.drawImage(bufferedImage, 0, 0, this);
		}
	}

	//何らかのイベント（ボタンを押したなど）が発生したら呼ばれる
	public void actionPerformed(ActionEvent e) {
		 String cmd = e.getActionCommand();
		if (cmd.equals("b1")){
			System.out.println("ボタン１が押された");
		}
	}
}
